==> bettertrims-4.0.0%2B1.21.1-fabric-sources.jar: com/bawnorton/bettertrims/client/tooltip/condition/predicate/BlockPredicateTooltip.java <==
package com.bawnorton.bettertrims.client.tooltip.condition.predicate;

import com.bawnorton.bettertrims.client.tooltip.util.Styler;
import com.bawnorton.bettertrims.client.tooltip.component.CompositeContainerComponent;
import com.bawnorton.bettertrims.client.tooltip.condition.LootConditionTooltips;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Optional;
import net.minecraft.class_2105;
import net.minecraft.class_2248;
import net.minecraft.class_4550;
import net.minecraft.class_4559;
import net.minecraft.class_638;
import net.minecraft.class_6885;
import net.minecraft.class_7924;

//? if >=1.21.8 {
/*import com.bawnorton.bettertrims.client.tooltip.condition.predicate.data.DataComponentMatchersTooltip;
import net.minecraft.advancements.critereon.DataComponentMatchers;
*///?}

public interface BlockPredicateTooltip {
	static void addToBuilder(class_638 level, class_4550 predicate, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		Optional<class_6885<class_2248>> blocks = predicate.comp_1732();
		if (blocks.isPresent()) {
			addBlocksToBuilder(level, blocks.orElseThrow(), state, builder);
		}

		//? if >=1.21.8 {
    /*DataComponentMatchers components = predicate.components();
    if (!components.isEmpty()) {
        addDataComponentMatchersToBuilder(level, components, state, builder);
    }
    *///?}

		Optional<class_2105> nbt = predicate.comp_1734();
		if (nbt.isPresent()) {
			addNbtPredicateToBuilder(level, nbt.orElseThrow(), state, builder);
		}

		Optional<class_4559> properties = predicate.comp_1733();
		if (properties.isPresent()) {
			addStatePropertiesPredicateToBuilder(level, properties.orElseThrow(), state, builder);
		}
	}

	static String key(String key) {
		return PredicateTooltip.key("block.%s".formatted(key));
	}

	static void addBlocksToBuilder(class_638 level, class_6885<class_2248> blocks, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		PredicateTooltip.addRegisteredElementsToBuilder(level, key("matches"), class_7924.field_41254, blocks, class_2248::method_9518, state, builder);
	}

	//? if >=1.21.8 {
  /*static void addDataComponentMatchersToBuilder(ClientLevel level, DataComponentMatchers components, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
      DataComponentMatchersTooltip.addToBuilder(level, components, state, builder);
  }
  *///?}

	static void addNbtPredicateToBuilder(class_638 level, class_2105 nbt, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		EntityPredicateTooltip.addNbtPredicateToBuilder(level, key("nbt"), nbt, state, builder);
	}

	static void addStatePropertiesPredicateToBuilder(class_638 level, class_4559 properties, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		List<StatePropertiesPredicate.PropertyMatcher> matchers = properties.properties();
		if (matchers.isEmpty()) {
			builder.component(CompositeContainerComponent.builder()
					.space()
					.translate(key("state_properties.any"), Styler::value)
					.build());
		} else {
			builder.component(CompositeContainerComponent.builder()
					.space()
					.cycle(propertyCycle -> {
						for (StatePropertiesPredicate.PropertyMatcher matcher : matchers) {
							CompositeContainerComponent.Builder propertyMatcherBuilder = CompositeContainerComponent.builder();
							Component matcherName = Styler.property(Component.literal(StringUtils.capitalize(matcher.name())));
							switch (matcher.valueMatcher()) {
								case StatePropertiesPredicate.ExactMatcher exact -> propertyMatcherBuilder.translate(
										key("state_property.exact"),
										Styler::condition,
										matcherName,
										Styler.value(Component.literal(StringUtils.capitalize(exact.value())))
								);
								case StatePropertiesPredicate.RangedMatcher ranged -> {
									String min = ranged.minValue().orElse(null);
									String max = ranged.maxValue().orElse(null);

									if (min == null && max == null) {
										propertyMatcherBuilder.translate(
												key("state_property.any"),
												Styler::condition,
												matcherName
										);
									} else if (min != null && max != null) {
										propertyMatcherBuilder.translate(
												key("state_property.between"),
												Styler::condition,
												matcherName,
												Styler.number(Component.literal(min)),
												Styler.number(Component.literal(max))
										);
									} else if (min != null) {
										propertyMatcherBuilder.translate(
												key("state_property.at_least"),
												Styler::condition,
												matcherName,
												Styler.number(Component.literal(min))
										);
									} else {
										propertyMatcherBuilder.translate(
												key("state_property.at_most"),
												Styler::condition,
												matcherName,
												Styler.number(Component.literal(max))
										);
									}
								}
								default -> propertyMatcherBuilder.translate(key("state_property.any"), Styler::condition, matcherName);
							}
							propertyCycle.component(propertyMatcherBuilder.build());
						}
					})
					.build());
		}
	}
}

==> bettertrims-4.0.2%2B1.21.10-fabric-sources.jar: com/bawnorton/bettertrims/client/tooltip/condition/predicate/BlockPredicateTooltip.java <==
package com.bawnorton.bettertrims.client.tooltip.condition.predicate;

import com.bawnorton.bettertrims.client.tooltip.util.Styler;
import com.bawnorton.bettertrims.client.tooltip.component.CompositeContainerComponent;
import com.bawnorton.bettertrims.client.tooltip.condition.LootConditionTooltips;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Optional;
import net.minecraft.class_10914;
import net.minecraft.class_2105;
import net.minecraft.class_2248;
import net.minecraft.class_4550;
import net.minecraft.class_4559;
import net.minecraft.class_638;
import net.minecraft.class_6885;
import net.minecraft.class_7924;
//? if >=1.21.8 {
import com.bawnorton.bettertrims.client.tooltip.condition.predicate.data.DataComponentMatchersTooltip;

public interface BlockPredicateTooltip {
	static void addToBuilder(class_638 level, class_4550 predicate, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		Optional<class_6885<class_2248>> blocks = predicate.comp_1732();
		if (blocks.isPresent()) {
			addBlocksToBuilder(level, blocks.orElseThrow(), state, builder);
		}

		//? if >=1.21.8 {
		class_10914 components = predicate.comp_3832();
		if (!components.method_68680()) {
			addDataComponentMatchersToBuilder(level, components, state, builder);
		}
		//?}

		Optional<class_2105> nbt = predicate.comp_1734();
		if (nbt.isPresent()) {
			addNbtPredicateToBuilder(level, nbt.orElseThrow(), state, builder);
		}

		Optional<class_4559> properties = predicate.comp_1733();
		if (properties.isPresent()) {
			addStatePropertiesPredicateToBuilder(level, properties.orElseThrow(), state, builder);
		}
	}

	static String key(String key) {
		return PredicateTooltip.key("block.%s".formatted(key));
	}

	static void addBlocksToBuilder(class_638 level, class_6885<class_2248> blocks, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		PredicateTooltip.addRegisteredElementsToBuilder(level, key("matches"), class_7924.field_41254, blocks, class_2248::method_9518, state, builder);
	}

	//? if >=1.21.8 {
	static void addDataComponentMatchersToBuilder(class_638 level, class_10914 components, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		DataComponentMatchersTooltip.addToBuilder(level, components, state, builder);
	}
	//?}

	static void addNbtPredicateToBuilder(class_638 level, class_2105 nbt, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		EntityPredicateTooltip.addNbtPredicateToBuilder(level, key("nbt"), nbt, state, builder);
	}

	static void addStatePropertiesPredicateToBuilder(class_638 level, class_4559 properties, LootConditionTooltips.State state, CompositeContainerComponent.Builder builder) {
		List<StatePropertiesPredicate.PropertyMatcher> matchers = properties.properties();
		if (matchers.isEmpty()) {
			builder.component(CompositeContainerComponent.builder()
					.space()
					.translate(key("state_properties.any"), Styler::value)
					.build());
		} else {
			builder.component(CompositeContainerComponent.builder()
					.space()
					.cycle(propertyCycle -> {
						for (StatePropertiesPredicate.PropertyMatcher matcher : matchers) {
							CompositeContainerComponent.Builder propertyMatcherBuilder = CompositeContainerComponent.builder();
							Component matcherName = Styler.property(Component.literal(StringUtils.capitalize(matcher.name())));
							switch (matcher.valueMatcher()) {
								case StatePropertiesPredicate.ExactMatcher exact -> propertyMatcherBuilder.translate(
										key("state_property.exact"),
										Styler::condition,
										matcherName,
										Styler.value(Component.literal(StringUtils.capitalize(exact.value())))
								);
								case StatePropertiesPredicate.RangedMatcher ranged -> {
									String min = ranged.minValue().orElse(null);
									String max = ranged.maxValue().orElse(null);

									if (min == null && max == null) {
										propertyMatcherBuilder.translate(
												key("state_property.any"),
												Styler::condition,
												matcherName
										);
									} else if (min != null && max != null) {
										propertyMatcherBuilder.translate(
												key("state_property.between"),
												Styler::condition,
												matcherName,
												Styler.number(Component.literal(min)),
												Styler.number(Component.literal(max))
										);
									} else if (min != null) {
										propertyMatcherBuilder.translate(
												key("state_property.at_least"),
												Styler::condition,
												matcherName,
												Styler.number(Component.literal(min))
										);
									} else {
										propertyMatcherBuilder.translate(
												key("state_property.at_most"),
												Styler::condition,
												matcherName,
												Styler.number(Component.literal(max))
										);
									}
								}
								default -> propertyMatcherBuilder.translate(key("state_property.any"), Styler::condition, matcherName);
							}
							propertyCycle.component(propertyMatcherBuilder.build());
						}
					})
					.build());
		}
	}
}
